/*
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * thrift - a lightweight cross-language rpc/serialization tool
 *
 * This file contains the main compiler engine for Thrift, which invokes the
 * scanner/parser to build the thrift object tree. The interface generation
 * code for each language lives in a file by the language name under the
 * generate/ folder, and all parse structures live in parse/
 *
 */

#include <string>
#include <vector>
#include <thrift/compiler/common.h>
#include <thrift/compiler/compiler.h>
#include <thrift/compiler/parse/parsing_driver.h>

using namespace apache::thrift::compiler;

int main(int argc, char** argv) {
  auto result = compile(std::vector<std::string>(argv, argv + argc));
  dump_diagnostics(result.diagnostics);
  return (int)result.retcode;
}
